@@ -22,7 +22,7 @@ import notification from 'ant-design-vue/es/notification'
 import { vueProps } from '@/vue-app'
 import router from '@/router'
 import store from '@/store'
-import { login, logout, api } from '@/api'
+import { oauthlogin, login, logout, api } from '@/api'
 import { i18n } from '@/locales'
 
 import {
@@ -36,7 +36,9 @@ import {
   HEADER_NOTICES,
   DOMAIN_STORE,
   DARK_MODE,
-  CUSTOM_COLUMNS
+  CUSTOM_COLUMNS,
+  OAUTH_DOMAIN,
+  OAUTH_PROVIDER
 } from '@/store/mutation-types'
 
 const user = {
@@ -61,9 +63,12 @@ const user = {
     loginFlag: false,
     logoutFlag: false,
     customColumns: {},
+    shutdownTriggered: false,
     twoFaEnabled: false,
     twoFaProvider: '',
-    twoFaIssuer: ''
+    twoFaIssuer: '',
+    customHypervisorName: 'Custom',
+    readyForShutdownPollingJob: ''
   },
 
   mutations: {
@@ -133,6 +138,9 @@ const user = {
       vueProps.$localStorage.set(CUSTOM_COLUMNS, customColumns)
       state.customColumns = customColumns
     },
+    SET_SHUTDOWN_TRIGGERED: (state, shutdownTriggered) => {
+      state.shutdownTriggered = shutdownTriggered
+    },
     SET_LOGOUT_FLAG: (state, flag) => {
       state.logoutFlag = flag
     },
@@ -147,6 +155,18 @@ const user = {
     },
     SET_LOGIN_FLAG: (state, flag) => {
       state.loginFlag = flag
+    },
+    SET_CUSTOM_HYPERVISOR_NAME (state, name) {
+      state.customHypervisorName = name
+    },
+    SET_READY_FOR_SHUTDOWN_POLLING_JOB: (state, job) => {
+      state.readyForShutdownPollingJob = job
+    },
+    SET_DOMAIN_USED_TO_LOGIN: (state, domain) => {
+      vueProps.$localStorage.set(OAUTH_DOMAIN, domain)
+    },
+    SET_OAUTH_PROVIDER_USED_TO_LOGIN: (state, provider) => {
+      vueProps.$localStorage.set(OAUTH_PROVIDER, provider)
     }
   },
 
@@ -201,6 +221,53 @@ const user = {
       })
     },
 
+    OauthLogin ({ commit }, userInfo) {
+      return new Promise((resolve, reject) => {
+        oauthlogin(userInfo).then(response => {
+          const result = response.loginresponse || {}
+          Cookies.set('account', result.account, { expires: 1 })
+          Cookies.set('domainid', result.domainid, { expires: 1 })
+          Cookies.set('role', result.type, { expires: 1 })
+          Cookies.set('timezone', result.timezone, { expires: 1 })
+          Cookies.set('timezoneoffset', result.timezoneoffset, { expires: 1 })
+          Cookies.set('userfullname', result.firstname + ' ' + result.lastname, { expires: 1 })
+          Cookies.set('userid', result.userid, { expires: 1 })
+          Cookies.set('username', result.username, { expires: 1 })
+          vueProps.$localStorage.set(ACCESS_TOKEN, result.sessionkey, 24 * 60 * 60 * 1000)
+          commit('SET_TOKEN', result.sessionkey)
+          commit('SET_TIMEZONE_OFFSET', result.timezoneoffset)
+
+          const cachedUseBrowserTimezone = vueProps.$localStorage.get(USE_BROWSER_TIMEZONE, false)
+          commit('SET_USE_BROWSER_TIMEZONE', cachedUseBrowserTimezone)
+          const darkMode = vueProps.$localStorage.get(DARK_MODE, false)
+          commit('SET_DARK_MODE', darkMode)
+          const cachedCustomColumns = vueProps.$localStorage.get(CUSTOM_COLUMNS, {})
+          commit('SET_CUSTOM_COLUMNS', cachedCustomColumns)
+
+          commit('SET_APIS', {})
+          commit('SET_NAME', '')
+          commit('SET_AVATAR', '')
+          commit('SET_INFO', {})
+          commit('SET_PROJECT', {})
+          commit('SET_HEADER_NOTICES', [])
+          commit('SET_FEATURES', {})
+          commit('SET_LDAP', {})
+          commit('SET_CLOUDIAN', {})
+          commit('SET_DOMAIN_STORE', {})
+          commit('SET_LOGOUT_FLAG', false)
+          commit('SET_2FA_ENABLED', (result.is2faenabled === 'true'))
+          commit('SET_2FA_PROVIDER', result.providerfor2fa)
+          commit('SET_2FA_ISSUER', result.issuerfor2fa)
+          commit('SET_LOGIN_FLAG', false)
+          notification.destroy()
+
+          resolve()
+        }).catch(error => {
+          reject(error)
+        })
+      })
+    },
+
     GetInfo ({ commit }, switchDomain) {
       return new Promise((resolve, reject) => {
         const cachedApis = switchDomain ? {} : vueProps.$localStorage.get(APIS, {})
@@ -278,6 +345,9 @@ const user = {
           if (result && result.defaultuipagesize) {
             commit('SET_DEFAULT_LISTVIEW_PAGE_SIZE', result.defaultuipagesize)
           }
+          if (result && result.customhypervisordisplayname) {
+            commit('SET_CUSTOM_HYPERVISOR_NAME', result.customhypervisordisplayname)
+          }
         }).catch(error => {
           reject(error)
         })
@@ -304,6 +374,11 @@ const user = {
           cloudianUrl = state.cloudian.url + 'logout.htm?redirect=' + encodeURIComponent(window.location.href)
         }
 
+        Object.keys(Cookies.get()).forEach(cookieName => {
+          Cookies.remove(cookieName)
+          Cookies.remove(cookieName, { path: '/client' })
+        })
+
         commit('SET_TOKEN', '')
         commit('SET_APIS', {})
         commit('SET_PROJECT', {})
@@ -331,11 +406,6 @@ const user = {
           }
         }).catch(() => {
           resolve()
-        }).finally(() => {
-          Object.keys(Cookies.get()).forEach(cookieName => {
-            Cookies.remove(cookieName)
-            Cookies.remove(cookieName, { path: '/client' })
-          })
         })
       })
     },
@@ -391,6 +461,15 @@ const user = {
         }).catch(error => {
           reject(error)
         })
+
+        api('listConfigurations', { name: 'hypervisor.custom.display.name' }).then(json => {
+          if (json.listconfigurationsresponse.configuration !== null) {
+            const config = json.listconfigurationsresponse.configuration[0]
+            commit('SET_CUSTOM_HYPERVISOR_NAME', config.value)
+          }
+        }).catch(error => {
+          reject(error)
+        })
       })
     },
     UpdateConfiguration ({ commit }) {
@@ -411,6 +490,9 @@ const user = {
     },
     SetLoginFlag ({ commit }, loggedIn) {
       commit('SET_LOGIN_FLAG', loggedIn)
+    },
+    SetCustomHypervisorName ({ commit }, name) {
+      commit('SET_CUSTOM_HYPERVISOR_NAME', name)
     }
   }
 }
